from playwright.sync_api import Page
from guara.transaction import AbstractTransaction


class OpenApp(AbstractTransaction):
    """
    Opens the App

    Args:
        with_url (str): The URL of the App

    Returns:
        str: the title of the App
    """

    def __init__(self, driver):
        super().__init__(driver)
        self._driver: Page

    def do(self, with_url):
        self._driver.goto(with_url)
        return self._driver.title()



import pytest
from playwright.sync_api import Page
from examples.web_ui.playwright.pages import home, setup, getting_started
from guara.transaction import Application
from guara import it


@pytest.fixture
def setup_method(page: Page):
    app = Application(page)
    app.at(
        setup.OpenApp,
        with_url="https://playwright.dev/",
    ).asserts(it.Contains, "Playwright")
    yield app


@pytest.mark.skip(
    "Check the requirements to run Playwright in"
    "   https://playwright.dev/python/docs/intro#installing-playwright-pytest"
)
def test_local_page_playwright(setup_method):
    dev_page: Application = setup_method
    dev_page.at(home.NavigateToGettingStarted).asserts(it.IsEqualTo, "Installation")
    dev_page.at(getting_started.NavigateToWritingTests).asserts(
        it.IsNotEqualTo, "Writing Tests"
    )



from selenium.webdriver.common.by import By
from guara.transaction import AbstractTransaction


class SubmitText(AbstractTransaction):
    """
    Submits the text

    Args:
        text (str): The text to be submited

    Returns:
        str: the label 'It works! {code}!'
    """

    def __init__(self, driver):
        super().__init__(driver)

    def do(self, text):
        text_box = self._driver.find_element(by=By.ID, value="input")
        submit_button = self._driver.find_element(by=By.CSS_SELECTOR, value="button")
        text_box.send_keys(text)
        submit_button.click()
        message = self._driver.find_element(by=By.ID, value="result")
        return message.text



from guara.asynchronous.transaction import AbstractTransaction


class OpenApp(AbstractTransaction):
    """
    Not Implemented as Selenium is not executed asynchronously.
    Use your preferable asynchronous Web Driver.
    For example: https://github.com/douglasdcm/caqui
    """

    def __init__(self, driver):
        super().__init__(driver)

    async def do(self, **kwargs):
        raise NotImplementedError(
            "Selenium does not support asynchronous execution."
            " Use your preferable async WebDriver. "
            " For example https://github.com/douglasdcm/caqui"
        )


class CloseApp(AbstractTransaction):
    """
    Not Implemented as Selenium is not executed asynchronously.
    Use your preferable asynchronous Web Driver.
    For example: https://github.com/douglasdcm/caqui
    """

    def __init__(self, driver):
        super().__init__(driver)

    async def do(self, **kwargs):
        raise NotImplementedError(
            "Selenium does not support asynchronous execution."
            " Use your preferable async WebDriver. "
            " For example https://github.com/douglasdcm/caqui"
        )



import logging
from typing import Any

LOGGER = logging.getLogger(__name__)


class IAssertion:
    def asserts(self, actual: Any, expected: Any) -> None:
        raise NotImplementedError

    def validates(self, actual, expected):
        LOGGER.info(f"Assertion '{self.__class__.__name__}'")
        try:
            result = self.asserts(actual, expected)
            LOGGER.info(f" actual:   '{actual}'")
            LOGGER.info(f" expected: '{expected}'")
            return result
        except Exception:
            LOGGER.error(f" actual:   '{actual}'")
            LOGGER.error(f" expected: '{expected}'")
            raise


class IsEqualTo(IAssertion):
    def asserts(self, actual, expected):
        assert actual == expected


class IsNotEqualTo(IAssertion):
    def asserts(self, actual, expected):
        assert actual != expected


class Contains(IAssertion):
    def asserts(self, actual, expected):
        assert expected in actual


class DoesNotContain(IAssertion):
    def asserts(self, actual, expected):
        assert expected not in actual


class HasKeyValue(IAssertion):
    """Checks whether the `actual` dictionary has the key and value
    set in `expected`. Returns when the first key-value pair is found and ignores
    the remaining ones.

    Args:
        actual (dict): the dictionary to be inspected
        expected (dict): the key-value pair to be found in `actual`
    """

    def asserts(self, actual, expected):
        for k, v in actual.items():
            if list(expected.keys())[0] in k and list(expected.values())[0] in v:
                return
        raise AssertionError


class MatchesRegex(IAssertion):
    """Checks whether the `expected` pattern matches the `actual` string

    Args:
        actual (str): the string to be inspected
        expected (str): the pattern to be found in `actual`. For example '(?:anoother){d}'
    """

    def asserts(self, actual, expected):
        import re

        if re.match(expected, actual):
            return
        raise AssertionError


class HasSubset(IAssertion):
    """Checks whether the `expected` list is a subset of `actual`

    Args:
        actual (list): the list to be inspected
        expected (list): the list to be found in `actual`.
    """

    def asserts(self, actual, expected):
        if set(expected).intersection(actual) == set(expected):
            return
        raise AssertionError


class IsSortedAs(IAssertion):
    """Checks whether the `actual` list is as `expected`

    Args:
        actual (list): the list to be inspected
        expected (list): the ordered list to compare againts `actual`.
    """

    def asserts(self, actual, expected):
        IsEqualTo().asserts(actual, expected)



import random
import pathlib
import pytest
import pytest_asyncio
from caqui import synchronous
from caqui.easy.capabilities import CapabilitiesBuilder

# imports from asynchrounous modules
from guara.asynchronous.transaction import Application
from guara.asynchronous import it

# `setup``is not the built-in transaction
from examples.web_ui.caqui_async.asynchronouos import setup, home
from examples.web_ui.caqui_async.constants import MAX_INDEX


# comment it to execute
@pytest.mark.skip(
    reason="before execute it start the driver as a service"
    "https://github.com/douglasdcm/caqui/tree/main?tab=readme-ov-file#simple-start"
)
class TestAsyncTransaction:
    # Set the fixtures as asynchronous
    @pytest_asyncio.fixture(loop_scope="function")
    async def setup_test(self):
        file_path = pathlib.Path(__file__).parent.parent.resolve()
        # This is how Caqui works
        # https://github.com/douglasdcm/caqui?tab=readme-ov-file#simple-start
        self._driver_url = "http://127.0.0.1:9999"
        capabilities = (
            CapabilitiesBuilder()
            .browser_name("chrome")
            .accept_insecure_certs(True)
            # comment it to see the UI of the browser
            .additional_capability(
                {"goog:chromeOptions": {"extensions": [], "args": ["--headless"]}}
            )
        ).build()
        self._session = synchronous.get_session(self._driver_url, capabilities)
        self._app = Application(self._session)

        # Notice the introduction of the method `perform`
        # It gets the list of coroutines and exeutes them in order
        await self._app.at(
            setup.OpenApp,
            with_session=self._session,
            connect_to_driver=self._driver_url,
            access_url=f"file:///{file_path}/sample.html",
        ).asserts(it.IsEqualTo, "Sample page").perform()
        yield
        await self._app.at(
            setup.CloseApp,
            with_session=self._session,
            connect_to_driver=self._driver_url,
        ).perform()

    async def _run_it(self):
        """Get all MAX_INDEX links from page and validates its text"""
        # arrange
        text = ["cheese", "selenium", "test", "bla", "foo"]
        text = text[random.randrange(len(text))]
        expected = []
        max_index = MAX_INDEX - 1
        for i in range(max_index):
            expected.append(f"any{i+1}.com")

        # act and assert
        # the method `perform` runs the coroutine related to GetAllLinks first and saves the
        # result for further asserition. Then, it runs the coroutine of `asserts` and asserts
        # the resuslt against the expected value
        await self._app.at(
            home.GetAllLinks,
            with_session=self._session,
            connect_to_driver=self._driver_url,
        ).asserts(it.IsEqualTo, expected).perform()

        # Does the same think as above, but asserts the items using the built-in method `assert`
        # arrange
        for i in range(max_index):

            # act
            actual = await self._app.at(
                home.GetNthLink,
                link_index=i + 1,
                with_session=self._session,
                connect_to_driver=self._driver_url,
            ).perform()

            # assert
            assert actual.result == f"any{i+1}.com"

    # both tests run in paralell
    # it is necessary to mark the test as async
    @pytest.mark.asyncio
    async def test_async_page_1(self, setup_test):
        await self._run_it()

    @pytest.mark.asyncio
    async def test_async_page_2(self, setup_test):
        await self._run_it()

    @pytest.mark.asyncio
    async def test_async_page_3(self, setup_test):
        await self._run_it()



from guara.transaction import AbstractTransaction
from examples.unit_test.todo_list.todo import ToDo


class Add(AbstractTransaction):
    def __init__(self, driver):
        super().__init__(driver)
        self._driver: ToDo

    def do(self, task):
        return self._driver.add_task(task)


class Remove(AbstractTransaction):
    def __init__(self, driver):
        super().__init__(driver)
        self._driver: ToDo

    def do(self, task):
        return self._driver.remove_task(task)


class ListTasks(AbstractTransaction):
    def __init__(self, driver):
        super().__init__(driver)
        self._driver: ToDo

    def do(self):
        return self._driver.list_tasks()


class PrintDict(AbstractTransaction):
    def __init__(self, driver):
        super().__init__(driver)
        self._driver: ToDo

    def do(self):
        return self._driver.to_dict()


class GetBy(AbstractTransaction):
    def __init__(self, driver):
        super().__init__(driver)
        self._driver: ToDo

    def do(self, index):
        return self._driver.get_by_index(index)



from guara.transaction import AbstractTransaction
from examples.unit_test.calculator.calculator import Calculator


class Add(AbstractTransaction):
    def __init__(self, driver):
        super().__init__(driver)
        self._driver: Calculator

    def do(self, a, b):
        return self._driver.add(a, b)


class Subtract(AbstractTransaction):
    def __init__(self, driver):
        super().__init__(driver)
        self._driver: Calculator

    def do(self, a, b):
        return self._driver.subtract(a, b)



from caqui import asynchronous
from guara.transaction import AbstractTransaction
from examples.web_ui.caqui_async.constants import MAX_INDEX


class GetNthLink(AbstractTransaction):
    """
    Get the nth link from the page

    Args:
        link_index (int): The index of the link
        with_session (object): The session of the Web Driver
        connect_to_driver (str): The URL to connect the Web Driver server

    Returns:
        str: The nth link
    """

    def __init__(self, driver):
        super().__init__(driver)

    async def do(
        self,
        link_index,
        with_session,
        connect_to_driver,
    ):
        locator_type = "xpath"
        locator_value = f"//a[@id='a{link_index}']"
        anchor = await asynchronous.find_element(
            connect_to_driver, with_session, locator_type, locator_value
        )
        return await asynchronous.get_text(connect_to_driver, with_session, anchor)


class GetAllLinks(AbstractTransaction):
    """
    Get the list of links from the page

    Args:
        with_session (object): The session of the Web Driver
        connect_to_driver (str): The URL to connect the Web Driver server

    Returns:
        str: The list of links
    """

    def __init__(self, driver):
        super().__init__(driver)

    async def do(self, with_session, connect_to_driver):
        links = []
        max_index = MAX_INDEX - 1

        for i in range(max_index):
            i += 1
            links.append(
                # Instead of duplicate the code it is possible to call transactions directly
                await GetNthLink(None).do(
                    link_index=i,
                    with_session=with_session,
                    connect_to_driver=connect_to_driver,
                )
            )
        return links



import pathlib
import random
from selenium import webdriver
from examples.web_ui.selenium.simple import home
from guara.transaction import Application
from guara import it
from guara import setup


class TestLocalTransaction:
    def setup_method(self, method):
        file_path = pathlib.Path(__file__).parent.parent.resolve()
        options = webdriver.ChromeOptions()
        options.add_argument("--headless=new")
        self._app = Application(webdriver.Chrome(options=options))
        self._app.at(
            setup.OpenApp,
            url=f"file:///{file_path}/sample.html",
            window_width=1094,
            window_height=765,
            implicitly_wait=0.5,
        ).asserts(it.IsEqualTo, "Sample page")

    def teardown_method(self, method):
        self._app.at(setup.CloseApp)

    def test_local_page(self):
        text = ["cheese", "selenium", "test", "bla", "foo"]
        text = text[random.randrange(len(text))]
        self._app.at(home.SubmitText, text=text).asserts(
            it.IsEqualTo, f"It works! {text}!"
        )
        self._app.at(home.SubmitText, text=text).asserts(it.IsNotEqualTo, "Any")



from datetime import datetime
from helium import start_chrome, go_to, get_driver, kill_browser
from guara.transaction import AbstractTransaction


class OpenApp(AbstractTransaction):
    """
    Opens the app

    Args:
        url (str): the path where the screenshot is saved.
    """

    def __init__(self, driver):
        super().__init__(driver)

    def do(self, url, headless=True):
        start_chrome(headless=headless)
        go_to(url)


class CloseApp(AbstractTransaction):
    """
    Closes the app and saves its screenshot (PNG)

    Args:
        screenshot_filename (str): the name of the screenshot file.
        Defaults to './captures/guara-{datetime.now()}.png'.
    """

    def __init__(self, driver):
        super().__init__(driver)

    def do(
        self,
        screenshot_filename="./captures/guara-capture",
    ):
        get_driver().save_screenshot(f"{screenshot_filename}-{datetime.now()}.png")
        kill_browser()



import logging
from typing import Any, NoReturn
from selenium.webdriver.remote.webdriver import WebDriver
from guara.it import IAssertion
from guara.utils import get_transaction_info

LOGGER = logging.getLogger(__name__)


class AbstractTransaction:
    def __init__(self, driver: WebDriver):
        self._driver = driver

    def do(self, **kwargs) -> Any | NoReturn:
        raise NotImplementedError


class Application:
    """This is the runner of the automation."""

    def __init__(self, driver):
        self._driver = driver
        self._result = None

    @property
    def result(self):
        return self._result

    def at(self, transaction: AbstractTransaction, **kwargs):
        """It executes the `do` method of each transaction"""

        LOGGER.info(f"Transaction '{get_transaction_info(transaction)}'")
        for k, v in kwargs.items():
            LOGGER.info(f" {k}: {v}")

        self._result = transaction(self._driver).do(**kwargs)
        return self

    def asserts(self, it: IAssertion, expected):
        """The `asserts` method receives a reference to an `IAssertion` instance.
        It implements the `Strategy Pattern (GoF)` to allow its behavior to change at runtime.
        It validates the result using the `asserts` method."""

        it().validates(self._result, expected)
        return self



import logging
from typing import Any, TYPE_CHECKING

if TYPE_CHECKING:
    from guara.asynchronous.transaction import AbstractTransaction

LOGGER = logging.getLogger(__name__)


class IAssertion:

    async def asserts(self, actual: "AbstractTransaction", expected: Any) -> None:
        raise NotImplementedError

    async def validates(self, actual, expected):
        try:
            await self.asserts(actual, expected)
        except Exception:
            LOGGER.error(f"actual:   '{actual.result}'")
            LOGGER.error(f"expected: '{expected}'")
            raise


class IsEqualTo(IAssertion):
    async def asserts(self, actual, expected):
        assert actual.result == expected


class IsNotEqualTo(IAssertion):
    async def asserts(self, actual, expected):
        assert actual.result != expected


class Contains(IAssertion):
    async def asserts(self, actual, expected):
        assert expected.result in actual


class DoesNotContain(IAssertion):
    async def asserts(self, actual, expected):
        assert expected.result not in actual



import random
import pathlib
import pytest
from selenium import webdriver

from guara.transaction import Application
from guara import it, setup

from examples.web_ui.caqui_async.constants import MAX_INDEX

# `setup` is not the built-in transaction
from examples.web_ui.caqui_async.synchronous import home


class TestSyncTransaction:
    # Set the fixtures as asynchronous
    @pytest.fixture(scope="function")
    def setup_test(self):
        file_path = pathlib.Path(__file__).parent.parent.resolve()
        options = webdriver.ChromeOptions()
        options.add_argument("--headless=new")
        self._app = Application(webdriver.Chrome(options=options))

        self._app.at(
            setup.OpenApp,
            url=f"file:///{file_path}/sample.html",
        ).asserts(it.IsEqualTo, "Sample page")
        yield
        self._app.at(
            setup.CloseApp,
        )

    def _run_it(self):
        """Get all MAX_INDEX links from page and validates its text"""
        # arrange
        text = ["cheese", "selenium", "test", "bla", "foo"]
        text = text[random.randrange(len(text))]
        expected = []
        max_index = MAX_INDEX - 1
        for i in range(max_index):
            expected.append(f"any{i+1}.com")

        # act and assert
        self._app.at(
            home.GetAllLinks,
        ).asserts(it.IsEqualTo, expected)

        # Does the same think as above, but asserts the items using the built-in method `assert`
        # arrange
        for i in range(max_index):

            # act
            result = self._app.at(
                home.GetNthLink,
                link_index=i + 1,
            ).result

            # assert
            it.IsEqualTo().asserts(result, f"any{i+1}.com")

    # both tests run in paralell
    # it is necessary to mark the test as async
    def test_sync_page_1(self, setup_test):
        self._run_it()

    def test_sync_page_2(self, setup_test):
        self._run_it()

    def test_sync_page_3(self, setup_test):
        self._run_it()



import logging
from typing import Any, NoReturn
from selenium.webdriver.remote.webdriver import WebDriver
from guara.asynchronous.it import IAssertion
from guara.utils import get_transaction_info

LOGGER = logging.getLogger(__name__)


class AbstractTransaction:
    def __init__(self, driver: WebDriver):
        self._driver = driver

    async def do(self, **kwargs) -> Any | NoReturn:
        raise NotImplementedError


class Application:
    """
    This is the runner of the automation.
    """

    def __init__(self, driver):
        self._driver = driver
        self._result = None
        self._coroutines = []
        self._TRANSACTION = "transaction"
        self._ASSERTION = "assertion"
        self._kwargs = None
        self._transaction_name = None
        self._it = None
        self._expected = None

    @property
    def result(self):
        return self._result

    def at(self, transaction: AbstractTransaction, **kwargs):
        """It executes the `do` method of each transaction"""

        self._kwargs = kwargs
        self._transaction_name = get_transaction_info(transaction)
        coroutine = transaction(self._driver).do(**kwargs)
        self._coroutines.append({self._TRANSACTION: coroutine})

        return self

    def asserts(self, it: IAssertion, expected):
        """The `asserts` method receives a reference to an `IAssertion` instance.
        It implements the `Strategy Pattern (GoF)` to allow its behavior to change at runtime.
        It validates the result using the `asserts` method."""

        self._it = it
        self._expected = expected
        coroutine = it().validates(self, expected)
        self._coroutines.append({self._ASSERTION: coroutine})

        return self

    async def perform(self) -> "Application":
        """Executes the coroutines in order and saves the result of the transaction
        in `result`"""
        for coroutine in self._coroutines:
            if coroutine.get(self._TRANSACTION):
                LOGGER.info(f"Transaction '{self._transaction_name}'")
                for k, v in self._kwargs.items():
                    LOGGER.info(f" {k}: {v}")
                self._result = await coroutine.get(self._TRANSACTION)
                continue

            LOGGER.info(f"Assertion '{self._it.__name__}'")
            LOGGER.info(f" actual:   '{self._result}'")
            LOGGER.info(f" expected: '{self._expected}'")
            await coroutine.get(self._ASSERTION)
        self._coroutines.clear()
        return self



from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.common.by import By
from guara.transaction import AbstractTransaction


class NavigateTo(AbstractTransaction):
    """
    Navigates to Home page

    Returns:
        str: The label of the page
    """

    def __init__(self, driver):
        super().__init__(driver)

    def do(self, **kwargs):
        self._driver.find_element(By.CSS_SELECTOR, ".navbar-brand > img").click()
        self._driver.find_element(By.CSS_SELECTOR, ".col-md-10").click()
        return self._driver.find_element(By.CSS_SELECTOR, "label:nth-child(1)").text


class ChangeToEnglish(AbstractTransaction):
    """
    Changes the content of the Home page to English

    Returns:
        str: The label of the page in English
    """

    def __init__(self, driver):
        super().__init__(driver)

    def do(self, **kwargs):
        self._driver.find_element(By.CSS_SELECTOR, "button:nth-child(2) > img").click()
        self._driver.find_element(By.CSS_SELECTOR, ".col-md-10").click()
        return self._driver.find_element(By.CSS_SELECTOR, "label:nth-child(1)").text


class ChangeToPortuguese(AbstractTransaction):
    """
    Changes the content of the Home page to Portuguese

    Returns:
        str: The label of the page in Portuguese
    """

    def __init__(self, driver):
        super().__init__(driver)

    def do(self, **kwargs):
        self._driver.find_element(
            By.CSS_SELECTOR, ".btn:nth-child(3) > button:nth-child(1) > img"
        ).click()
        self._driver.find_element(By.CSS_SELECTOR, ".col-md-10").click()
        return self._driver.find_element(By.CSS_SELECTOR, "label:nth-child(1)").text


class Search(AbstractTransaction):
    """
    Searches for a given text

    Args:
        text (str): the text to be searched for
        wait_for (str): the text to present after the search
    Returns:
        str: The result with the first similarity of the search
    """

    def __init__(self, driver):
        super().__init__(driver)

    def fill_text(self, text):
        self._driver.find_element(By.ID, "message_field").click()
        self._driver.find_element(By.ID, "message_field").send_keys(text)

    def select_search(self):
        raise NotImplementedError

    def wait_search(self, wait_for):
        self._driver.find_element(By.ID, "send_message").click()
        WebDriverWait(self._driver, 30).until(
            expected_conditions.text_to_be_present_in_element(
                (By.CSS_SELECTOR, ".row:nth-child(1) > .col-md-2 > p"),
                wait_for,
            )
        )
        return self._driver.find_element(
            By.CSS_SELECTOR, ".row:nth-child(1) > .col-md-2 > p"
        ).text

    def do(self, text, wait_for):
        self.fill_text(text)
        self.select_search()
        return self.wait_search(wait_for)


class DoRestrictedSearch(Search):
    """
    Searches for a given text in restricted mode

    Args:
        text (str): the text to be searched for
        wait_for (str): the text to present after the search
    Returns:
        str: The result with the first similarity of the search
    """

    def __init__(self, driver):
        super().__init__(driver)

    def select_search(self):
        self._driver.find_element(By.ID, "cond_and").click()


class DoExpandedSearch(Search):
    """
    Searches for a given text in expanded mode

    Args:
        text (str): the text to be searched for
        wait_for (str): the text to present after the search
    Returns:
        str: The result with the first similarity of the search
    """

    def __init__(self, driver):
        super().__init__(driver)

    def select_search(self):
        self._driver.find_element(By.ID, "cond_or").click()



from helium import find_all, write, click, S, Text
from guara.transaction import AbstractTransaction


class SubmitText(AbstractTransaction):
    """
    Submits the text

    Args:
        text (str): The text to be submited

    Returns:
        str: the label 'It works! {code}!'
    """

    def __init__(self, driver):
        super().__init__(driver)

    def do(self, text):
        TEXT = '//*[@id="input"]'
        BUTTON_TEST = "button"
        text_field = find_all(S(TEXT))[0]
        write(text, text_field)
        click(find_all(S(BUTTON_TEST))[0])
        return Text("It works!").value



from browserist import Browser
from guara.transaction import AbstractTransaction


class SubmitText(AbstractTransaction):
    """
    Submits the text

    Args:
        text (str): The text to be submited

    Returns:
        str: the label 'It works! {code}!'
    """

    def __init__(self, driver):
        super().__init__(driver)
        self._driver: Browser

    def do(self, text):
        TEXT = '//*[@id="input"]'
        BUTTON_TEST = '//*[@id="button"]'
        RESULT = '//*[@id="result"]'
        self._driver.input.value(TEXT, text)
        self._driver.click.button(BUTTON_TEST)
        return self._driver.get.text(RESULT)



from selenium.webdriver.common.by import By
from guara.transaction import AbstractTransaction


class NavigateTo(AbstractTransaction):
    """
    Navigates to Contact page

    Returns:
        str: Paragraph informing the contacts
    """

    def __init__(self, driver):
        super().__init__(driver)

    def do(self, **kwargs):
        self._driver.find_element(By.CSS_SELECTOR, ".btn:nth-child(6) img").click()
        self._driver.find_element(By.CSS_SELECTOR, ".container:nth-child(3)").click()
        return self._driver.find_element(By.CSS_SELECTOR, "p:nth-child(1)").text



from playwright.sync_api import Page
from guara.transaction import AbstractTransaction


class NavigateToGettingStarted(AbstractTransaction):
    """
    Navigates to Getting Started page

    Returns:
        str: the heading 'Installation'
    """

    def __init__(self, driver):
        super().__init__(driver)
        self._driver: Page

    def do(self, **kwargs):
        self._driver.get_by_role("link", name="Get started").click()
        return self._driver.get_by_role("heading", name="Installation").text_content()
